/* 
 * File   : ffield_main.cpp
 * Version: 2008-06-06
 *
 * Copyright (C) 2008 Jochen Peters
 *
 * dev@emirp.net
 * http://www.emirp.net/
 * 
 * This file is part of FissionField and FissionField2x.
 *
 * FissionField(2x) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * FissionField(2x) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with FissionField(2x). If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <iostream>
#include <unistd.h>
#include <SDL/SDL.h>
//#include <SDL/SDL_audio.h>
#include "SFont.h"
#include "sdl_textmenu.hpp"
#include "sdl_ffield.hpp"
#include "ffield_player.hpp"


/*******************************
 * Defines & Global variables  *
 *******************************/

#define SDL_FFIELD_VERSION "Version: 1.0  (2008-05-12)"
#define SDL_FFIELD_WEBSITE "www.emirp.net"

/* Background */
#define BG_RED 153
#define BG_GREEN 153
#define BG_BLUE 153

/* Platform specific settings */
#ifdef GP2X 
	/* Settings for GP2X */
	#define SCREEN_WIDTH 320
	#define SCREEN_HEIGHT 240
	#define SCREEN_DEPTH 16
	#define SCREEN_BORDER 10
	#define POSX_GAMEINFO (210 + SCREEN_BORDER)
	#define POSY_GAMEINFO (70 + SCREEN_BORDER)
	#define TILESIZE 20
	#define TEXTLINE_HEIGHT 10
#else
	/* Settings for PC */
	#define SCREEN_WIDTH 640
	#define SCREEN_HEIGHT 480
	#define SCREEN_DEPTH 16
	#define SCREEN_BORDER 30
	#define POSX_GAMEINFO (420 + SCREEN_BORDER)
	#define POSY_GAMEINFO (100 + SCREEN_BORDER)
	#define TILESIZE 40
	#define TEXTLINE_HEIGHT 20
#endif

/* Define GP2X button events */
#define GP2X_BUTTON_UP              (0)
#define GP2X_BUTTON_DOWN            (4)
#define GP2X_BUTTON_LEFT            (2)
#define GP2X_BUTTON_RIGHT           (6)
#define GP2X_BUTTON_UPLEFT          (1)
#define GP2X_BUTTON_UPRIGHT         (7)
#define GP2X_BUTTON_DOWNLEFT        (3)
#define GP2X_BUTTON_DOWNRIGHT       (5)
#define GP2X_BUTTON_CLICK           (18)
#define GP2X_BUTTON_A               (12)
#define GP2X_BUTTON_B               (13)
#define GP2X_BUTTON_X               (14)
#define GP2X_BUTTON_Y               (15)
#define GP2X_BUTTON_L               (10)
#define GP2X_BUTTON_R               (11)
#define GP2X_BUTTON_START           (8)
#define GP2X_BUTTON_SELECT          (9)
#define GP2X_BUTTON_VOLUP           (16)
#define GP2X_BUTTON_VOLDOWN         (17)

#define FONT_FILE "bitmaps/font.bmp"
#define LOGO_FILE "bitmaps/logo.bmp"
	
/* Variables */
SDL_Surface* screen = NULL;
SDL_Joystick* joystick = NULL;
//SoundData soundToPlay;
int soundVolume = 80;



/*******************************
 * Play game                   *
 *******************************/


void displayScore(SFont_Font* font, int red_squares, int red_points, int blue_squares, int blue_points) {

	SDL_Rect area;
	area.w = SCREEN_WIDTH - POSX_GAMEINFO;
	area.h = TEXTLINE_HEIGHT * 4;
	area.x = POSX_GAMEINFO;
	area.y = POSY_GAMEINFO;

	/* Clear area */
	SDL_FillRect(screen, &area, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE)); // set background color

	/* Write */
	char tmpChar[256];
	unsigned int ypos = POSY_GAMEINFO;
	SFont_Write(screen, font, POSX_GAMEINFO, ypos, "Score");
	ypos += TEXTLINE_HEIGHT;
	sprintf(tmpChar, "Red: %d / %d", red_squares, red_points);
	SFont_Write(screen, font, POSX_GAMEINFO, ypos, tmpChar);
	ypos += TEXTLINE_HEIGHT;
	sprintf(tmpChar, "Blue: %d / %d", blue_squares, blue_points);
	SFont_Write(screen, font, POSX_GAMEINFO, ypos, tmpChar);

	/* Update screen */
	SDL_UpdateRect(screen, area.x, area.y, area.w, area.h);

}

void displayStatus(SFont_Font* font, char* line1, char* line2) {

	SDL_Rect area;
	area.w = SCREEN_WIDTH - POSX_GAMEINFO;
	area.h = TEXTLINE_HEIGHT * 3;
	area.x = POSX_GAMEINFO;
	area.y = POSY_GAMEINFO + TEXTLINE_HEIGHT * 5;

	/* Clear area */
	SDL_FillRect(screen, &area, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE)); // set background color

	/* Write */
	unsigned int ypos = POSY_GAMEINFO + TEXTLINE_HEIGHT * 5;
	SFont_Write(screen, font, POSX_GAMEINFO, ypos, line1);
	ypos += TEXTLINE_HEIGHT;
	SFont_Write(screen, font, POSX_GAMEINFO, ypos, line2);

	/* Update screen */
	SDL_UpdateRect(screen, area.x, area.y, area.w, area.h);

}

void displayGameinfo(SFont_Font* font) {

	SDL_Rect area;
	area.w = SCREEN_WIDTH - POSX_GAMEINFO;
	area.h = SCREEN_HEIGHT;
	area.x = POSX_GAMEINFO;
	area.y = 0;

	/* Clear screen */
	SDL_FillRect(screen, &area, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE)); // set background color

	/* Display logo */
	SDL_Rect blitting_rect;
	SDL_Surface* logo = SDL_LoadBMP(LOGO_FILE);
	if(!logo) {
		std::cerr << "An error occured while loading bitmap(s). Abort." << std::endl;
		exit(1);
	}
	SDL_SetColorKey(logo, SDL_SRCCOLORKEY, SDL_MapRGB(logo->format, 255, 0, 255)); // set transparent color
	blitting_rect.w = logo->w;
	blitting_rect.h = logo->h;
	blitting_rect.x = POSX_GAMEINFO - 3;
	blitting_rect.y = SCREEN_BORDER;
	SDL_BlitSurface(logo, NULL, screen, &blitting_rect);

	/* Display text */
	displayScore(font, 0, 0, 0, 0);
	displayStatus(font, "Red's turn", "");
#ifdef GP2X
	SFont_Write(screen, font, POSX_GAMEINFO, SCREEN_BORDER+TILESIZE*10-TEXTLINE_HEIGHT, "Press SELECT to exit.");
#else
	SFont_Write(screen, font, POSX_GAMEINFO, SCREEN_BORDER+TILESIZE*10-TEXTLINE_HEIGHT, "Press ESC to exit.");
#endif

	/* Update screen */
	SDL_UpdateRect(screen, area.x, area.y, area.w, area.h);

}

void playGame(int fieldSize, AtomvsPlayer* redPlayer, AtomvsPlayer* bluePlayer, bool mouse, bool sound) {

	/* Load font */
	SDL_Surface* fontBitmap = SDL_LoadBMP(FONT_FILE);
	SFont_Font* font = SFont_InitFont(fontBitmap);
	if(!font) {
		std::cerr << "An error occured while loading font. Abort." << std::endl;
		exit(1);
	}

	/* Clear screen */
	SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE)); // set background color

	/* Create AtomicField */
	int posx_field = SCREEN_BORDER + (10-fieldSize)*TILESIZE/2;
	int posy_field = posx_field;
	sdlAtomicField field(fieldSize, posx_field, posy_field, sound);
	sdlFieldCursor cursor(&field);

	/* Set volume */
	if(sound) {
		Mix_VolumeChunk(field.clickSound, soundVolume);
	}

	/* Display info text */
	displayGameinfo(font);

	/* Prepare players */
	AtomvsPlayer* players[2];
	players[RED] = redPlayer;
	players[BLUE] = bluePlayer;

	/* User input loop */
	enum color teamToMove = RED;
	bool quit = false;
	bool runningGame = true;
	while(!quit) {

		bool makeMove = false;
		bool validMove = false;
		int row;
		int col;

		/* Computer move */
		if(runningGame && players[teamToMove] != NULL) {
			SDL_Delay(200);
			players[teamToMove]->makeMove(&field, &row, &col);
			validMove = field.makeMove(row, col, teamToMove);
		}

		/* Show cursor if needed */
		if(!mouse && runningGame && players[teamToMove]==NULL) {
			cursor.setVisible(true);
		}

		SDL_Event event;
		while(SDL_PollEvent(&event)) {

			Uint8* keys;
			Uint8 joystickButton = 255;
			switch(event.type) {
				case SDL_QUIT:
					exit(0);
					break;
				
				case SDL_KEYDOWN:
				case SDL_JOYBUTTONDOWN:
					keys = SDL_GetKeyState(NULL); // get keys
					if(event.type == SDL_JOYBUTTONDOWN) {
						joystickButton = event.jbutton.button; // get joystick button
					}

					if(keys[SDLK_ESCAPE] || joystickButton==GP2X_BUTTON_SELECT) {
						quit = true;
					}
#ifdef GP2X
					/* Volume control for GP2X */
					else if(joystickButton==GP2X_BUTTON_VOLDOWN) {
						if(sound && soundVolume >= 10) {
							soundVolume -= 10;
							Mix_VolumeChunk(field.clickSound, soundVolume);
						}
					}
					else if(joystickButton==GP2X_BUTTON_VOLUP) {
						if(sound && soundVolume <=90) {
							soundVolume += 10;
							Mix_VolumeChunk(field.clickSound, soundVolume);
						}
					}
#endif
					else if(!mouse && runningGame && players[teamToMove]==NULL && (keys[SDLK_RIGHT] || joystickButton==GP2X_BUTTON_RIGHT)) {
						cursor.moveRight();
					}
					else if(!mouse && runningGame && players[teamToMove]==NULL && (keys[SDLK_DOWN] || joystickButton==GP2X_BUTTON_DOWN)) {
						cursor.moveDown();
					}
					else if(!mouse && runningGame && players[teamToMove]==NULL && (keys[SDLK_LEFT] || joystickButton==GP2X_BUTTON_LEFT)) {
						cursor.moveLeft();
					}
					else if(!mouse && runningGame && players[teamToMove]==NULL && (keys[SDLK_UP] || joystickButton==GP2X_BUTTON_UP)) {
						cursor.moveUp();
					}
					else if(!mouse && runningGame && players[teamToMove]==NULL && (keys[SDLK_RETURN] || joystickButton==GP2X_BUTTON_A || joystickButton==GP2X_BUTTON_B)) {
						/* User move */
						row = cursor.getRow();
						col = cursor.getCol();
						makeMove = true;
					}
					break;

				case SDL_MOUSEBUTTONDOWN:

					/* User move */
					if(mouse && runningGame && players[teamToMove] == NULL) {
						row = field.getRow(event.button.y);
						col = field.getCol(event.button.x);
						makeMove = true;
					}
					break;

				default:
					// do nothing
					break;

			}
		}

		/* Make move */
		if(makeMove && row >= 0 && col >= 0) {
			validMove = field.makeMove(row, col, teamToMove);
			makeMove = false;
		}

		/* Update scoreboard, change player to move and check for winner */
		if(validMove) {

			/* DEBUG: START */
			//field.printField();
			//std::cout.flush();
			/* DEBUG: END */

			char status_label1[SDL_TEXTMENU_MAXLENGTH];
			char status_label2[SDL_TEXTMENU_MAXLENGTH];
							
			/* Whose turn? */
			if(teamToMove == RED) {
				teamToMove = BLUE;
				sprintf(status_label1, "Blue's turn");
			}
			else {
				teamToMove = RED;
				sprintf(status_label1, "Red's turn");
			}

			/* Computer player? */
			if(players[teamToMove] == NULL) {
				sprintf(status_label2, "");
			}
			else {
				sprintf(status_label2, "Thinking...");
			}
								
			/* Is there a winner? */
			enum color winner = field.checkForWinner();
			if(winner == RED) {
				runningGame = false;
				sprintf(status_label1, "RED WON!");
				sprintf(status_label2, "");
			}
			else if(winner == BLUE) {
				runningGame = false;
				sprintf(status_label1, "BLUE WON!");
				sprintf(status_label2, "");
			}

			/* DEBUG: START */
			//if(winner != NONE) {
			//	field.printField();
			//	std::cout.flush();
			//}
			/* DEBUG: END */

			/* Update score */
			displayScore(font, field.getScore(RED), field.getNumParticles(RED), field.getScore(BLUE), field.getNumParticles(BLUE));
					
			/* Display status */
			displayStatus(font, status_label1, status_label2);

		}
		
	}

	/* Free font */
	SFont_FreeFont(font);
}



/*******************************
 * Main Menu                   *
 *******************************/


void displayMainMenu(sdlTextMenu* menu) {

	/* Load font */
	SDL_Surface* fontBitmap = SDL_LoadBMP(FONT_FILE);
	SFont_Font* font = SFont_InitFont(fontBitmap);
	if(!font) {
		std::cerr << "An error occured while loading font. Abort." << std::endl;
		exit(1);
	}

	/* Clear screen */
	SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE)); // set background color

	/* Display logo */
	SDL_Rect blitting_rect;
	SDL_Surface* logo = SDL_LoadBMP(LOGO_FILE);
	if(!logo) {
		std::cerr << "An error occured while loading bitmap(s). Abort." << std::endl;
		exit(1);
	}
	SDL_SetColorKey(logo, SDL_SRCCOLORKEY, SDL_MapRGB(logo->format, 255, 0, 255)); // set transparent color
	blitting_rect.w = logo->w;
	blitting_rect.h = logo->h;
	blitting_rect.x = SCREEN_BORDER + 17;
	blitting_rect.y = SCREEN_BORDER;
	SDL_BlitSurface(logo, NULL, screen, &blitting_rect);

	/* Display text menu */
	menu->displayMenu();

	/* Display info */
	SFont_Write(screen, font, SCREEN_BORDER+20, SCREEN_HEIGHT-SCREEN_BORDER-30, SDL_FFIELD_VERSION);
	SFont_Write(screen, font, SCREEN_WIDTH-SCREEN_BORDER-20-SFont_TextWidth(font, SDL_FFIELD_WEBSITE), SCREEN_HEIGHT-SCREEN_BORDER-30, SDL_FFIELD_WEBSITE);

	/* Update screen */
	SDL_UpdateRect(screen, 0, 0, 0, 0);

	/* Free font */
	SFont_FreeFont(font);

}


void mainMenu() {

	/* Game options */
	/* Player */
	const char* option_player[] = {"Human", "Computer 1", "Computer 2", "Computer 3", "Computer 4"}; // 1=Random, 2=MaxFields, 3=MinFields, 4=MaxParticles
	const int optionnum_player = 5;
	int optionindex_redplayer = 0;
	int optionindex_blueplayer = 4;
	/* Boardsize */
	const int option_size[] = {6, 8, 10};
	const int optionnum_size = 3;
	int optionindex_size = 2;
	/* Sound */
#ifdef GP2X 
	bool option_mouse = false;
#else
	bool option_mouse = true;
#endif
	bool option_sound = true;

	/* Menu item indices */
	const int menuindex_redplayer = 0;
	const int menuindex_blueplayer = 1;
	const int menuindex_boardsize = 2;
	const int menuindex_mouse = 3;
	const int menuindex_sound = 4;
	const int menuindex_startgame = 5;
	const int menuindex_exit = 6;

	/* Set up text menu */
	unsigned int ypos = POSY_GAMEINFO;
	sdlTextMenu menu(screen, 7, 5, FONT_FILE);
	menu.setItem(menuindex_redplayer, "Red Player:  Human", SCREEN_BORDER+20, ypos);
	ypos += TEXTLINE_HEIGHT;
	menu.setItem(menuindex_blueplayer, "Blue Player:  Computer 4", SCREEN_BORDER+20, ypos);
	ypos += TEXTLINE_HEIGHT * 2;
	menu.setItem(menuindex_boardsize, "Boardsize:  10x10", SCREEN_BORDER+20, ypos);
	ypos += TEXTLINE_HEIGHT;
#ifdef GP2X 
	menu.setItem(menuindex_mouse, "Controls:  Cursor", SCREEN_BORDER+20, ypos);
#else 
	menu.setItem(menuindex_mouse, "Controls:  Mouse", SCREEN_BORDER+20, ypos);
#endif
	ypos += TEXTLINE_HEIGHT;
	menu.setItem(menuindex_sound, "Sound:  on", SCREEN_BORDER+20, ypos);
	ypos += TEXTLINE_HEIGHT * 2;
	menu.setItem(menuindex_startgame, "Start Game", SCREEN_BORDER+20, ypos);
	ypos += TEXTLINE_HEIGHT * 2;
	menu.setItem(menuindex_exit, "Exit", SCREEN_BORDER+20, ypos);

	displayMainMenu(&menu);

	/* User input loop */
	bool quit = false;
	while(!quit) {

		SDL_Event event;
		while(SDL_PollEvent(&event)) {

			Uint8* keys;
			Uint8 joystickButton = 255;
			switch(event.type) {

				case SDL_QUIT:
					exit(0);
					break;

				case SDL_KEYDOWN:
				case SDL_JOYBUTTONDOWN:

					keys = SDL_GetKeyState(NULL); // get keys
					if(event.type == SDL_JOYBUTTONDOWN) {
						joystickButton = event.jbutton.button; // get joystick button
					}

					if(keys[SDLK_UP] || joystickButton==GP2X_BUTTON_UP) {
						menu.activatePrev();
					}
					if(keys[SDLK_DOWN] || joystickButton==GP2X_BUTTON_DOWN) {
						menu.activateNext();
					}
					if(keys[SDLK_LEFT] || joystickButton==GP2X_BUTTON_LEFT) {
						// do nothing
					}
					if(keys[SDLK_RIGHT] || joystickButton==GP2X_BUTTON_RIGHT) {
						// do nothing
					}
					if(keys[SDLK_RETURN] || joystickButton==GP2X_BUTTON_A || joystickButton==GP2X_BUTTON_B) {
						
						char option_label[SDL_TEXTMENU_MAXLENGTH];
						switch(menu.getActiveIndex()) {

							case menuindex_redplayer:
								optionindex_redplayer++;
								if(optionindex_redplayer > optionnum_player-1) {
									optionindex_redplayer = 0;
								}
								sprintf(option_label, "Red Player:  %s", option_player[optionindex_redplayer]);
								menu.updateItem(menuindex_redplayer, option_label, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE));
								break;

							case menuindex_blueplayer:
								optionindex_blueplayer++;
								if(optionindex_blueplayer > optionnum_player-1) {
									optionindex_blueplayer = 0;
								}
								sprintf(option_label, "Blue Player:  %s", option_player[optionindex_blueplayer]);
								menu.updateItem(menuindex_blueplayer, option_label, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE));
								break;

							case menuindex_boardsize:
								optionindex_size++;
								if(optionindex_size > optionnum_size-1) {
									optionindex_size = 0;
								}
								sprintf(option_label, "Boardsize:  %dx%d", option_size[optionindex_size], option_size[optionindex_size]);

								menu.updateItem(menuindex_boardsize, option_label, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE));
								break;

							case menuindex_mouse:
#ifndef GP2X
								if(option_mouse) {
									option_mouse = false;
									sprintf(option_label, "Controls:  Cursor");
								}
								else {
									option_mouse = true;
									sprintf(option_label, "Controls:  Mouse");
								}
								menu.updateItem(menuindex_mouse, option_label, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE));
#endif
								break;

							case menuindex_sound:
								if(option_sound) {
									option_sound = false;
									sprintf(option_label, "Sound:  off");
								}
								else {
									option_sound = true;
									sprintf(option_label, "Sound:  on");
								}
								menu.updateItem(menuindex_sound, option_label, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE));
								break;

							case menuindex_startgame:
								{
								/* Set up players */
								AtomvsPlayer* redPlayer = NULL;
								AtomvsPlayer* bluePlayer = NULL;
								if(optionindex_redplayer == 1) {
									redPlayer = new RandomPlayer(RED);
								}
								else if(optionindex_redplayer == 2) {
									redPlayer = new MaxFieldsPlayer(RED);
								}
								else if(optionindex_redplayer == 3) {
									redPlayer = new MinFieldsPlayer(RED);
								}
								else if(optionindex_redplayer == 4) {
									redPlayer = new MaxParticlesPlayer(RED);
								}
								if(optionindex_blueplayer == 1) {
									bluePlayer = new RandomPlayer(BLUE);
								}
								else if(optionindex_blueplayer == 2) {
									bluePlayer = new MaxFieldsPlayer(BLUE);
								}
								else if(optionindex_blueplayer == 3) {
									bluePlayer = new MinFieldsPlayer(BLUE);
								}
								else if(optionindex_blueplayer == 4) {
									bluePlayer = new MaxParticlesPlayer(BLUE);
								}

								/* Start game */
								playGame(option_size[optionindex_size], redPlayer, bluePlayer, option_mouse, option_sound);

								/* Delete players */
								delete redPlayer;
								delete bluePlayer;

								/* Restore menu */
								displayMainMenu(&menu);
								}
								break;

							case menuindex_exit:
								quit = true;
								break;

							default:
								// do nothing
								break;
						}

					}
					break;

				default:
					// do nothing
					break;

			}
		}
	}

}


/*******************************
 * main()                      *
 *******************************/

int main(int argc, char *argv[]) {

	/* Init SDL */
	if(SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO) == -1) {
		std::cerr << SDL_GetError() << std::endl;
		exit(1);
	}
	atexit(SDL_Quit); // function from stdlib.h: SDL_Quit is executed at end of program

	/* Open joystick if available */
	if(SDL_NumJoysticks() > 0) {
		joystick = SDL_JoystickOpen(0);
	}

	/* Screen setup */
	screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_DEPTH, SDL_SWSURFACE);
	if(screen == NULL) {
		std::cerr << SDL_GetError() << std::endl;
		exit(1);
	}
	SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, BG_RED, BG_GREEN, BG_BLUE)); // set background color

#ifdef GP2X
	SDL_ShowCursor(SDL_DISABLE);
#endif

	/* Sound setup
	SDL_AudioSpec audio;
    audio.freq = 22050;
    audio.format = AUDIO_S16;
    audio.channels = 2;
    audio.samples = 512;
    audio.callback = mixAudio;
    audio.userdata = NULL;
	*/

    /* Open audio device
    if(SDL_OpenAudio(&audio, NULL) < 0) {
		std::cerr << SDL_GetError() << std::endl;
        exit(1);
    }
    SDL_PauseAudio(0);
	*/

	/* Open audio device */
 	if(Mix_OpenAudio(22050, AUDIO_S16, 2, 128) != 0) {
		std::cerr << "Unable to initialize audio: " << Mix_GetError() << std::endl;
		exit(1);
	}

	/* Empty event queue */
	SDL_Event event;
	while(SDL_PollEvent(&event));

	/* Show main menu */
	mainMenu();

	/* Clean up */
    //SDL_CloseAudio();
    Mix_CloseAudio();
	if(SDL_JoystickOpened(0)) {
	    SDL_JoystickClose(joystick);
	}
	SDL_Quit();

#ifdef GP2X
	/* Return to GP2X menu */
	chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
#endif

	return 0;

}

